﻿
namespace XamDataGridSearchEngine
{
    using System.Collections.Generic;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Data;
    using System.Windows.Media.Animation;
    using System.ComponentModel;
    using Infragistics.Windows;
    using Infragistics.Windows.DataPresenter;

    /// <summary>
    /// Interaction logic for UserControl1.xaml
    /// </summary>
    [DesignTimeVisible(false)]
    public partial class SearchBar : UserControl
    {
        private XamDataGrid grid;
        private List<Record> resultRecords;
        private int currentIndex;
        private bool isOpen = false;

        public SearchBar()
        {
            this.InitializeComponent();
            this.Loaded += new RoutedEventHandler(UserControl_Loaded);
        }


        private void Button_Click(object sender, RoutedEventArgs e)
        {
            if (isOpen)
            {
                Storyboard closeBoard = TryFindResource("CloseSearchBar") as Storyboard;
                closeBoard.Begin();

                searchBox.Text = "";

                isOpen = false;
            }
            else
            {
                Storyboard openBoard = TryFindResource("OpenSearchBar") as Storyboard;
                openBoard.Begin();

                isOpen = true;
            }
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            grid = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(XamDataGrid), false) as XamDataGrid;

            if (grid != null)
            {
                Binding tagBinding = new Binding();
                tagBinding.Source = searchBox;
                tagBinding.Path = new PropertyPath("Text");
                tagBinding.Mode = BindingMode.TwoWay;

                grid.SetBinding(XamDataGrid.TagProperty, tagBinding);
            }
            resultRecords = new List<Record>();
        }

        private void searchBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            string searchStr = (sender as TextBox).Text.ToLower();

            foreach (Record rec in resultRecords)
            {
                rec.Tag = "";
            }

            resultRecords.Clear();

            if (searchStr.Equals(""))
            {
                return;
            }


            if (grid != null && grid.Records is DataRecordCollection)
            {
                foreach (DataRecord record in grid.Records)
                {
                    foreach (Cell cell in record.Cells)
                    {
                        if (cell.Value != null)
                        {
                            string cellString = cell.Value.ToString().ToLower();

                            if (cellString.Contains(searchStr))
                            {
                                record.Tag = "match";
                                resultRecords.Add(record);
                                break;
                            }
                        }
                    }

                    if (record.HasChildren)
                    {
                        AddMatchingRecords(record.ChildRecords, resultRecords, searchStr);
                    }
                }
            }

            if (grid != null && grid.Records is GroupByRecordCollection)
            {
                foreach (GroupByRecord gr in grid.Records)
                {
                    AddMatchingRecords(gr, resultRecords, searchStr);
                }
            }

            if (resultRecords.Count > 0)
            {
                currentIndex = 0;
                grid.BringRecordIntoView(resultRecords[currentIndex]);
            }
        }

        public void AddMatchingRecords(GroupByRecord gr, List<Record> recList, string text)
        {
            if (gr.HasChildren)
            {

                foreach (Record record in gr.ChildRecords)
                {
                    if (record is DataRecord)
                    {
                        foreach (Cell cell in (record as DataRecord).Cells)
                        {
                            string cellValue = cell.Value.ToString().ToLower();
                            if (cellValue.Contains(text))
                            {
                                record.Tag = "match";
                                recList.Add(record);
                            }
                        }
                    }

                    if (record is GroupByRecord)
                    {
                        AddMatchingRecords(record as GroupByRecord, recList, text);
                    }

                }

            }
        }

        public void AddMatchingRecords(RecordCollectionBase gr, List<Record> recList, string text)
        {

            foreach (Record record in gr)
            {
                if (record is DataRecord)
                {
                    foreach (Cell cell in (record as DataRecord).Cells)
                    {
                        string cellValue = cell.Value.ToString().ToLower();
                        if (cellValue.Contains(text))
                        {
                            record.Tag = "match";
                            recList.Add(record);
                        }
                    }
                }

                if (record.HasChildren)
                {
                    if (record is DataRecord)
                    {
                        AddMatchingRecords((record as DataRecord).ChildRecords, recList, text);
                    }
                    if (record is ExpandableFieldRecord)
                    {
                        AddMatchingRecords((record as ExpandableFieldRecord).ChildRecords, recList, text);
                    }
                }

            }
        }

        private void next_Click(object sender, RoutedEventArgs e)
        {
            if (currentIndex < resultRecords.Count - 1)
            {
                currentIndex++;
                grid.BringRecordIntoView(resultRecords[currentIndex]);
                grid.ActiveRecord = resultRecords[currentIndex];
                ((grid.ActiveRecord).ParentRecord).IsExpanded = !((grid.ActiveRecord).ParentRecord).IsExpanded;
            }
            else
            {
                MessageBox.Show("No More Results Found");
            }
        }

        private void prev_Click(object sender, RoutedEventArgs e)
        {
            if (currentIndex > 0)
            {
                currentIndex--;
                grid.BringRecordIntoView(resultRecords[currentIndex]);
                grid.ActiveRecord = resultRecords[currentIndex];
                ((grid.ActiveRecord).ParentRecord).IsExpanded = !((grid.ActiveRecord).ParentRecord).IsExpanded;
            }
            else
            {
                MessageBox.Show("No More Results Found");
            }
        }

        public void OpenCloseSearchBar(bool openSearchbar)
        {
            isOpen = openSearchbar;

            if (isOpen)
            {
                Storyboard openBoard = TryFindResource("OpenSearchBar") as Storyboard;
                openBoard.Begin();

                isOpen = true;
            }
            else
            {
                Storyboard closeBoard = TryFindResource("CloseSearchBar") as Storyboard;
                closeBoard.Begin();

                searchBox.Text = "";

                isOpen = false;
            }
        }

    }
}